package rocketBugsRender;
/*
	A_Rod... very lightweight, for painting only
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class A_Rod extends Player {					
	double radius = Env.fActinDiameter/2;
	double length, ADPPiLength, ADPLength;	// overall length and ADP-Pi, ADP lengths
	int actinCount, ADPPiCount, ADPCount;	// number of different sorts of f-actin
	Pt3D end1,end2;						// endpts of the rod
	double helixAng;						// defines the starting angle for filament helix
	boolean plusCapped, minusCapped;		// flags to indicate if ends are capped
	boolean tethered;
	Pt3D tetherPt;							// store location on surface of Listeria where tethered
	// some declarations related to painting of the rod
	Pt3D ptA = new Pt3D(); 
	Pt3D ptB = new Pt3D();
	Pt3D dR = new Pt3D();
	Pixel pixA = new Pixel();
	Pixel pixB = new Pixel();
	Pixel pixAp = new Pixel();
	Pixel pixBp = new Pixel();
	double thMon, thLine;			// angles for helix blob and line painting, respectively
	double bMonLoc, bLineLoc;		// location along rod used for helix blob and line painting, repectively
	boolean oddMon;				// boolean to specify odd or even monomer number
	
	public A_Rod () {}
	
	public void initializeFromFileInfo (Pt3D end1, Pt3D end2, Pt3D angle, int actinCount, int ADPPiCount, int ADPCount, double helixAng, byte plusCapped, byte minusCapped) {
		this.actinCount = actinCount;
		this.ADPPiCount = ADPPiCount;
		this.ADPCount = ADPCount;
		this.end1 = end1;
		this.end2 = end2;
		this.angle = angle;
		this.initialize();
		if (plusCapped == 1) { this.plusCapped = true; } else { this.plusCapped = false; }
		if (minusCapped == 1) { this.minusCapped = true; } else { this.minusCapped = false; }
		/*if (actACount > 0) {
			this.tethered = true;
		} else {
			this.tethered = false;
		}*/
		this.helixAng = helixAng;
	}
	
	public static A_Rod InitializeFromFileInfo (Pt3D end1, Pt3D end2, Pt3D angle, int actinCount, int ADPPiCount, int ADPCount, double helixAng, byte plusCapped, byte minusCapped) {
		A_Rod rod = new A_Rod();
		rod.actinCount = actinCount;
		rod.ADPPiCount = ADPPiCount;
		rod.ADPCount = ADPCount;
		rod.end1 = end1;
		rod.end2 = end2;
		rod.angle = angle;
		rod.initialize();
		if (plusCapped == 1) { rod.plusCapped = true; } else { rod.plusCapped = false; }
		if (minusCapped == 1) { rod.minusCapped = true; } else { rod.minusCapped = false; }
		/*if (actACount > 0) {
			rod.tethered = true;
		} else {
			rod.tethered = false;
		}*/
		rod.helixAng = helixAng;
		return rod;
	}
	
	public void initialize () {
		// calculate cos and sin of Euler angles only once during each time step
		cosAng.cosine(angle);
		sinAng.sine(angle);
		// find the transformation matrices at this time step
		transMat ();
		// find the unit vector that defines the orientation of the rod
		uVec.xToX(this, xUnitVector);
		t1Vec.xToX(this, yUnitVector); 
		t2Vec.cross(uVec, t1Vec);
	}
	
	public void mapActinLocation (HistogramPlus hp, Listeria lis) {
		Pt3D curPt = new Pt3D();
		Pt3D fromLisCen = new Pt3D();
		for (int i=0;i<actinCount;i++) {
			curPt.add(end1,i*Env.lengthPerActin,uVec);
			fromLisCen.sub(curPt,lis.coord);
			fromLisCen.XTox(lis);	// now in listeria local coords
			hp.addValue(fromLisCen.x);			// add to monomer count at coordinate val along listeria body axis
		}
	}
	
	public boolean intersectsPlane (Pt3D normalToPlane, Pt3D pointInPlane) {
		double uVal = -1;		// set multiplier to any value outside 0 to 1 interval
		double denom = Pt3D.Dot(normalToPlane, Pt3D.Sub(end2,end1));
		double numer = Pt3D.Dot(normalToPlane, Pt3D.Sub(pointInPlane,end1));
		if (denom != 0) {
			uVal = numer/denom;
		}
		if ((uVal >= 0) & (uVal <= 1)) {	// the line segment that is this rod intersects the plane
			return true;
		} else {
			return false;
		}
	}
	
	public void showPtToPt (Graphics g, double scale, PinCam cam, Pt3D ptA, Pt3D ptB) {
		if (Env.usePinCam) {
			pixA = cam.ptToPix(ptA, scale);
			pixB = cam.ptToPix(ptB, scale);
		} else {
			pixA.ptToPix(ptA, scale, cam.offset, cam.e);
			pixB.ptToPix(ptB, scale, cam.offset, cam.e);
		}
		if ((pixA.show) | (pixB.show)) { Pixel.drawLine(g,pixA,pixB); }
	}
	
	public void showHelixMonomers (Graphics g, double scale, PinCam cam, int monomers) {
		Color curColor = g.getColor();
		double R = Env.helicalBlobOffset;
		double dth = Env.helixAngleInc;
		double dx = Env.helixXInc;
		Pt3D bLinePt = new Pt3D ();
		int blobSize = (int)(Env.helixBlobDiam*scale);
		int halfBlobSize = (int)(blobSize/2);
		for (int i = 0; i < monomers; i++) {
			bLinePt.add(end1, bMonLoc, uVec);
			dR.add(R*Math.cos(thMon), t1Vec, R*Math.sin(thMon), t2Vec);
			ptA.add(bLinePt, dR);
			if (Env.usePinCam) {
				pixA = cam.ptToPix(ptA, scale);
			} else {
				pixA.ptToPix(ptA, scale, cam.offset, cam.e);
			}
			g.fillOval(pixA.x - halfBlobSize, pixA.y - halfBlobSize, blobSize, blobSize);
			g.setColor(Env.actinOutlineColor);
			g.drawOval(pixA.x - halfBlobSize, pixA.y - halfBlobSize, blobSize, blobSize);
			g.setColor(curColor);
			thMon += dth;
			bMonLoc += dx;
		} 
	}
	
	public void showHelixLines (Graphics g, double scale, PinCam cam, int monomers) {
		double R = Env.fActinDiameter/2;
		double dth = Env.helixAngleInc;
		double dx = Env.helixXInc;
		Pt3D bLinePt = new Pt3D ();
		for (int i = 0; i < monomers; i++) {
			bLinePt.add(end1, bLineLoc, uVec);
			dR.add(R*Math.cos(thLine), t1Vec, R*Math.sin(thLine), t2Vec);
			if (oddMon) {
				ptA.add(bLinePt, dR);
				pixA.ptToPix(ptA, scale, cam.offset, cam.e);
				if (bLineLoc > dx) { Pixel.drawLine (g, pixAp, pixA); }
				pixAp.copy(pixA);
				oddMon = false;
			} else {
				ptB.add(bLinePt, dR);
				pixB.ptToPix(ptB, scale, cam.offset, cam.e);
				if (bLineLoc > dx) { Pixel.drawLine (g, pixBp, pixB); }
				pixBp.copy(pixB);
				oddMon = true;
			}
			thLine += dth;
			bLineLoc += dx;
		}		
	}
	
	public void showHelix (Graphics g, double scale, PinCam cam, int monomers) {
		if (Env.drawHelicalLines) { showHelixLines (g, scale, cam, monomers); }
		if (Env.drawEachMonomer) { showHelixMonomers (g, scale, cam, monomers); }
	}
		
	
	public void showYourself (Graphics g, double scale, PinCam cam) {
		thMon = helixAng;		// set initial value for helical angles
		thLine = helixAng;
		bMonLoc = 0;			// reset filament location to zero
		bLineLoc = 0;			
		oddMon = true;		// set odd monomer to true
		
		if (minusCapped) {	// draw minus caps first for better look at side-branches
			g.setColor(Env.minusCapColor);
			int capSize = (int)(Env.minusCapDiam*scale);
			int halfSize = (int)(capSize/2);
			pixA.ptToPix(end1, scale, cam.offset, cam.e);
			g.fillOval(pixA.x-halfSize, pixA.y-halfSize, capSize, capSize);
		}	
		
		
		if (Env.drawActinHelix) {
			if (Env.showHydrolysisStages) {	// show ATP, ADP-Pi, and ADP sections of actin
				// *** ADP section
				g.setColor(Env.ADPActinColor);
				showHelix (g, scale, cam, ADPCount);
				// *** ADP-Pi section
				g.setColor(Env.ADPPiActinColor);
				showHelix (g, scale, cam, ADPPiCount);
				// *** ATP section
				g.setColor(Env.ATPActinColor);
				showHelix (g, scale, cam, actinCount - ADPCount - ADPPiCount);
			} else {
				g.setColor(Env.ATPActinColor);
				showHelix (g, scale, cam, actinCount);
			}
		} else {
			length = actinCount*Env.lengthPerActin;
			ADPPiLength = ADPPiCount*Env.lengthPerActin;
			ADPLength = ADPCount*Env.lengthPerActin;
			if (Env.showHydrolysisStages) {	// show ATP, ADP-Pi, and ADP sections of actin
				//*** ADP section
				g.setColor(Env.ADPActinColor);
				Pt3D ADPPt = Pt3D.Add(end1, ADPLength, uVec);
				showPtToPt (g, scale, cam, end1, ADPPt);
				//*** ADP-Pi section
				g.setColor(Env.ADPPiActinColor);
				Pt3D ADPPiPt = Pt3D.Add(ADPPt, ADPPiLength, uVec);
				showPtToPt (g, scale, cam, ADPPt, ADPPiPt);
				//*** ATP section
				g.setColor(Env.ATPActinColor);
				showPtToPt (g, scale, cam, ADPPiPt, end2);
			} else {
				g.setColor(Env.ATPActinColor);
				showPtToPt (g, scale, cam, end1, end2);
			}
		}
		
		if (plusCapped) {
			g.setColor(Env.plusCapColor);
			int capSize = (int)(Env.plusCapDiam*scale);
			int halfSize = (int)(capSize/2);
			pixA.ptToPix(end2, scale, cam.offset, cam.e);
			g.fillOval(pixA.x-halfSize, pixA.y-halfSize, capSize, capSize);
		}
		
		
		if (tethered) {
			/*g.setColor(Env.tetherColor);
			pixA.ptToPix(end2, scale, cam.offset, cam.e);
			pixB.ptToPix(tetherPt, scale, cam.offset, cam.e);
			Pixel.drawLine(g, pixA, pixB);
			
			// draw the ActA to which this filament is bound
			g.setColor(Env.boundActAColor);
			pixA.ptToPix(tetherPt,scale,cam.offset,cam.e);
			int ActAPixels = (int) (scale*0.01);
			g.drawOval(pixA.x-ActAPixels/2, pixA.y-ActAPixels/2, ActAPixels, ActAPixels);*/
		}	
	}		
	
}