package rocketBugsRender;
/*
	Line... stores a line equation	y=mx+b
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;

public class Line {
	double m;	// slope
	double b;	// intercept
	int start;	// start index for this line
	int stop;	// stop index for this line
	int ptCt; // stop-start
	double sigA,sigB,chiSq,fitQ;
	boolean show = true;
	
	public Line () {}
	
	public Line (double m, double b, int start, int stop) {
		this.m = m;
		this.b = b;
		this.start = start;
		this.stop = stop;
		ptCt = stop-start;
	}
	
	public Line (double m, double b, int start, int stop, double sigA, double sigB, double chiSq, double fitGoodness) {
		this.m = m;
		this.b = b;
		this.start = start;
		this.stop = stop;
		this.sigA = sigA;
		this.sigB = sigB;
		this.chiSq = chiSq;
		this.fitQ = fitGoodness;
		ptCt = stop-start;
	}
	
	public double getVal (double x) {
		// send back value of y for a particular x
		return m*x + b;
	}
	
	public double getVal (int x) { 
		// if x given as int, cast to double and use getVal(double)
		return getVal((double)x);
	}
	
	public double [] getAllVals () {
		// send back all values assuming increment of 1 for x
		int numVals = stop-start+1;
		double [] vals = new double [numVals];
		for (int i=0;i<numVals;i++) {
			vals[i] = getVal(start+i);
		}
		return vals;
	}
	
	public double getAveVal (int first, int last) {
		double aveVal = 0;
		for (int i=first;i<=last;i++) {
			aveVal += getVal(i);
		}
		aveVal = aveVal/(last-first+1);
		return aveVal;
	}
	
	public double getAveVal () {
		return getAveVal(start, stop);
	}
	
	public double getStartVal () {
		return getVal(start);
	}
	
	public double getStopVal () {
		return getVal(stop);
	}
			
	public double getTimeSlope (DataColumn timeCol) {
		double startVal = getVal(start);
		double stopVal = getVal(stop);
		double startTime = timeCol.data[start];
		double stopTime = timeCol.data[stop];
		return (stopVal-startVal)/(stopTime-startTime);
	}

	public String reportLine () {
		String eq = "line is y = " + String.valueOf(m) + "x + " + String.valueOf(b) + " ; chiSq is " + String.valueOf(chiSq);
		return eq;
	}


}
		