package rocketBugsRender;
/*
	StepAnalyzer... class that analyzes for steps
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;

public class StepAnalyzer extends JFrame implements ChangeListener,ActionListener,KeyListener {
	DataColumn inCol;
	LineColumn outCol;
	LineColumn stepCol;
	DataColumn timeCol;
	DataColumn tethersCol;
	DataSet inSet;
	DataColumn [] eventCols;
	HistogramPlus [] eventHists;
	int rowCt;
	int avStub = 5;
	PlotFrame parentFrame;
	Dimension frameDim = new Dimension(250,50);
	static int yDisp = 1;	// use to offset on screen different instances
	DecimalFormat chiOutFormat = new DecimalFormat ("0.#E0");
	boolean fitLap = false;
	boolean plateaud = false;
	boolean remote = false;
	int startIndex,stopIndex;
	int startTime = 0;
	int stopTime = (int)1e6;	// big number
	
	//File saving
	JFileChooser plateauSave = new JFileChooser();
	
	// gui control of analysis
	static final int startTimeRange = 10; // (up to 10 seconds)
	static final int stopTimeRange = 60; // (up to 60 seconds)
		//** global slope vals
	static double maxSlope = 0;
	static double minSlope = 0;
	static double deltaSlope = 0;
		//** chiSq vals
	static final int minInt = 0;	// multiplier for chiSq
	static final int maxInt = 10;
	static final int minExp = 6;	// range of exponents for chiSq
	static final int maxExp = 15;	
	int chiInt = 1;
	int chiExp = -7;
	double curChiSq = chiInt*Math.pow(10,chiExp);
		//** lap vals
	static final int minLap = 0;
	static final int maxLap = 50;
	int curLap = 0;
		//** max velocity (slope) for plateaus
	static final int minVeloc = 0;
	static final int maxVeloc = 200;
	double curVeloc = 80; // nanometers/s
		// ** min step	** minimum pause to keep
	static final int minMinS = 1;
	static final int maxMinS = 100;
	int curMinPause = 10;
		// ** disjoint step ** distance defining separate plateaus
	static final int minDisJ = 0;
	static final int maxDisJ = 20;
	int curDisJInt = 10;	// multiplier
	static final double disJStep = 0.0001;  // .1 nm
	double curDisJ = curDisJInt*disJStep;  // range from 0 to 2 nm
		//** angle base vals
	static final int minAngBase = -90;
	static final int maxAngBase = 90;
	double curAngleBase = 0;
		//** angle tol vals
	double curAngleTol = 0;
		//** gui
	static String startTimeString = "Start Time =";
	static String stopTimeString = "Stop Time = ";
	static String chiString = "ChiSq = ";
	static String lapString = "Lap = " ;
	static String velocString = "Max Veloc. = " ;
	static String minStepString = "Min Pause = " ;
	static String disJDistString = "Disjoint Dist. = " ;
	static String angBaseString = "Base Angle = " ;
	static String angTolString = "Angle Tol. = " ;
	// panels and buttons
	JPanel timePanel,sliPanel,labPanel,butPanel;
	JButton mergeB, filterPlatsB, eventHistoB, platsHistoB;
	
	JSlider startTimeSlide = new JSlider (JSlider.HORIZONTAL,0,startTimeRange,0);
	JSlider stopTimeSlide = new JSlider (JSlider.HORIZONTAL,0,stopTimeRange,stopTimeRange);
	JSlider chiSqSlide = new JSlider (JSlider.HORIZONTAL,minInt,maxInt,chiInt);
	JSlider chiSqExpSlide = new JSlider (JSlider.HORIZONTAL,minExp,maxExp,-chiExp);
	JSlider lapSlide = new JSlider (JSlider.HORIZONTAL,minLap,maxLap,curLap);
	JSlider velocSlide = new JSlider (JSlider.HORIZONTAL,minVeloc, maxVeloc, (int)curVeloc);
	JSlider minStepSlide = new JSlider (JSlider.HORIZONTAL,minMinS, maxMinS, curMinPause);
	JSlider disJSlide = new JSlider (JSlider.HORIZONTAL,minDisJ, maxDisJ, curDisJInt);
	JSlider angBaseSlide = new JSlider (JSlider.HORIZONTAL,minAngBase,maxAngBase,(int)curAngleBase);
	JSlider angTolSlide = new JSlider (JSlider.HORIZONTAL,0,90,(int)curAngleTol);
	
	JLabel startTimeLabel = new JLabel (startTimeString + String.valueOf(startTime));
	JLabel stopTimeLabel = new JLabel (stopTimeString + "eof");
	JLabel chiSqLabel = new JLabel(chiString + String.valueOf(chiOutFormat.format(chiInt*Math.pow(10,chiExp))));
	JLabel lapLabel = new JLabel(lapString + String.valueOf(curLap));
	JLabel velocLabel = new JLabel(velocString + String.valueOf(curVeloc) + " nm/s");
	JLabel minStepLabel = new JLabel(minStepString + String.valueOf(curMinPause));
	JLabel disJLabel = new JLabel(disJDistString + String.valueOf(chiOutFormat.format(curDisJ*1000)) + " nm");
	JLabel angBaseLabel = new JLabel(angBaseString + String.valueOf(chiOutFormat.format(curAngleBase)));
	JLabel angTolLabel = new JLabel(angTolString + String.valueOf(chiOutFormat.format(curAngleTol)));
	
	
	public StepAnalyzer (DataColumn inCol, PlotFrame parentFrame, DataColumn timeCol, boolean remote) {
		this.parentFrame = parentFrame;
		setColumns (inCol,timeCol);
		setStartAndStop();
		this.remote = remote;
		
		makeGuiParts();
		
		if (!remote) {
			// frame settings
			setTitle ("Lines " + inCol.set.name + ":" + inCol.name);
			setSize(getMySize());
			setResizable(true);
			setLocation(getMyLocation());
			
			addKeyListener(this);
		}
				
		initialize();
	}
	
	public StepAnalyzer () {
		this.remote = true;
		makeGuiParts();
	}
	
	public void nullLinks() {
		if (inCol != null) { inCol.nullLinks(); inCol = null; }
		if (outCol != null) { outCol.nullLinks(); outCol = null; }
		if (stepCol != null) { stepCol.nullLinks(); stepCol = null; }
		if (timeCol != null) { timeCol.nullLinks(); timeCol = null; }
		if (inSet != null) { inSet.nullLinks(); inSet = null; }
		for (int i=0;i<eventCols.length;i++) {
			if (eventCols[i] != null) { eventCols[i].nullLinks(); eventCols[i] = null; }
		}
		for (int i=0;i<eventHists.length;i++) {
			if (eventHists[i] != null) { eventHists[i].nullLinks(); eventHists[i] = null; }
		}
	}
	
	public void nullLineColumns() {
		if (outCol != null) { outCol.nullLinks(); outCol = null; }
		if (stepCol != null) { stepCol.nullLinks(); stepCol = null; }
	}
	
	public void makeGuiParts() {
		// listeners
		startTimeSlide.addChangeListener(this);
		stopTimeSlide.addChangeListener(this);
		chiSqSlide.addChangeListener(this);
		chiSqExpSlide.addChangeListener(this);
		lapSlide.addChangeListener(this);
		velocSlide.addChangeListener(this);
		minStepSlide.addChangeListener(this);
		disJSlide.addChangeListener(this);
		angBaseSlide.addChangeListener(this);
		angTolSlide.addChangeListener(this);
		
		Dimension slideDim = new Dimension (200,18);
		Dimension halfSlideDim = new Dimension (100,18);
		startTimeSlide.setPreferredSize(slideDim);
		stopTimeSlide.setPreferredSize(slideDim);
		chiSqSlide.setPreferredSize(halfSlideDim);
		chiSqExpSlide.setPreferredSize(halfSlideDim);
		lapSlide.setPreferredSize(slideDim);
		velocSlide.setPreferredSize(slideDim);
		minStepSlide.setPreferredSize(slideDim);
		disJSlide.setPreferredSize(slideDim);
		angBaseSlide.setPreferredSize(slideDim);
		angTolSlide.setPreferredSize(slideDim);
		
		// make gui parts
		makeTimePanel();
		makeSliderPanel();
		makeLabelPanel();
		makeButtonPanel();
	}
	
	public void setStartAndStop (int start, int stop) {
		startTime = start;
		stopTime = stop;
		setStartAndStop();
	}
	
	public void setStartAndStop () {
		for (int i=0;i<rowCt;i++) {
			if (timeCol.data[i] >= startTime) {
				startIndex = i;
				break;
			}
		}
		
		for (int i=rowCt-1;i>0;i+=-1) {
			if (timeCol.data[i] < stopTime) {
				stopIndex = i;
				break;
			}
		}
		
		//talk ("Using data from " + String.valueOf(startTime) + " secs (index=" + String.valueOf(startIndex) + ")");
		//talkln (" to " + String.valueOf(stopTime) + " secs (index=" + String.valueOf(stopIndex) + ")");
	}
	
	public void setColumns (DataColumn inCol, DataColumn timeCol) {
		this.rowCt = inCol.rows;
		this.inCol = inCol;
		this.timeCol = timeCol;
		nullLineColumns();
		// out column
		this.outCol = new LineColumn(rowCt);
		outCol.setName("Fit:"+ inCol.getSetName() + ":" + inCol.getName());
		outCol.setParentCol(inCol);
		// step column
		this.stepCol = new LineColumn(rowCt);
		stepCol.setName("Steps:" + inCol.getSetName() + ":" + inCol.getName());
		stepCol.setParentCol(inCol);
	}
	
	public void setTethersCol (DataColumn tethersCol) {
		this.tethersCol = tethersCol;
	}
	
	public class RetObj {
		// this is the object passed by from line-line and line-point intersect tests
		Pt3D conPt1, conPt2;
		double conDist;
		
		public RetObj (Pt3D pt1, Pt3D pt2, double dist) {
			conPt1 = pt1;
			conPt2 = pt2;
			conDist = dist;
		}
	}
	
	public class LineMap {
		// this object holds the lines that map any pt
		int lineInc = 10;
		int lineCt = 0;
		Line [] lines = new Line[1];	
		
		public LineMap () {
		}
		
		public void addLine(Line newLine) {
			if (lineCt == lines.length-1) { lines = moreLines(); }
			lines[lineCt] = newLine;
			lineCt++;
		}
		
		public Line [] moreLines () {
			Line [] mLines = new Line[lines.length + lineInc];
			for (int i=0;i<lineCt;i++) {
				mLines[i] = lines[i];
			}
			return mLines;
		}
		
		public double aveVal(double pt) {
			if (lineCt == 0) { return Double.NaN; }
			double val=0;
			for (int i=0;i<lineCt;i++) {
				val+= lines[i].getVal(pt);
			}
			val = val/lineCt;
			return val;
		}
		
		public int getLineCt() {
			return lineCt;
		}
		
		public boolean bigJump (double pt) {
			if (lineCt == 0) { return false; }
			// checks for a jump in value over which we break up the plateau
			double tol = curDisJ; // microns
			double min = lines[0].getVal(pt);
			double max = lines[0].getVal(pt);
			for (int i=0;i<lineCt;i++) {
				double curVal = lines[i].getVal(pt);
				if (curVal < min) { min = curVal; }
				if (curVal > max) { max = curVal; }
				if ((max-min) > tol) { return true; }
			}
			return false;
		}
		
	}
	
	private void rePlot () {
		if (!remote) {
			parentFrame.plotAll();
		}
	}
	
	private Point getMyLocation() {
		Point parPt = parentFrame.getLocation();
		Dimension parDim = parentFrame.getSize();
		Point myPt = new Point();
		int yInc = 10;
		myPt.x = parPt.x;
		myPt.y = parPt.y + parDim.height + yDisp*yInc;
		yDisp++;
		return myPt;
	}
	
	private Dimension getMySize() {
		Dimension mySize = new Dimension();
		Dimension sliSize = sliPanel.getPreferredSize();
		Dimension labSize = labPanel.getPreferredSize();
		Dimension butSize = butPanel.getPreferredSize();
		mySize.width = sliSize.width + labSize.width + butSize.width + 40;
		mySize.height = sliSize.height + 40;
		return mySize;
	}
	
	public void initialize () {
		if (!remote) {
			getContentPane().setLayout(new BorderLayout());
			JPanel cenPanel = new JPanel();
			cenPanel.setLayout(new FlowLayout());
			cenPanel.add (sliPanel);
			cenPanel.add (labPanel);
			cenPanel.add (butPanel);
			getContentPane().add(timePanel,BorderLayout.NORTH);
			getContentPane().add(cenPanel,BorderLayout.CENTER);
			show();
		}
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Work")) {
			work();
			updateGUI();
			rePlot();
		}
		
		if (act.equals("Trim")) {
			trimLines();
			rePlot();
		}
		
		if (act.equals("Merge")) {
			mergeLines();
			updateGUI();
			rePlot();
		}
		
		if (act.equals("Plateaus")) {
			updatePlateaus();
			updateGUI();
			rePlot();
		}
		
		if (act.equals("Filter Plateaus")) {
			if (plateaud) {
				filterPlateaus();
				rePlot();
			}
		}
		
		if (act.equals("Events Histo")) {
			new EventTracker(getInSet(),stepCol,timeCol);
		}
		/*if (act.equals("Events Histo")) {
			makeEventHisto();
		}*/
		
		if (act.equals("Plats Histo")) {
			makePlateauHisto();
		}
	}
	
	public void keyReleased (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider)e.getSource();
        
        if (source.equals(startTimeSlide)) {
        	startTime = (int)(startTimeSlide.getValue());
        	startTimeLabel.setText(startTimeString + String.valueOf(startTime));
        	if (!source.getValueIsAdjusting()) { setStartAndStop(); }
        }
        
        if (source.equals(stopTimeSlide)) {
        	stopTime = (int)(stopTimeSlide.getValue());
        	stopTimeLabel.setText(stopTimeString + String.valueOf(stopTime));
        	if (stopTime == stopTimeRange) {
        		stopTime = (int)1e6;
        		stopTimeLabel.setText(stopTimeString + "eof");
        	}
        	if (!source.getValueIsAdjusting()) { setStartAndStop(); }
        }
        
        if ((source.equals(chiSqSlide)) | (source.equals(chiSqExpSlide))) {
        	chiInt = (int)chiSqSlide.getValue();
        	chiExp = -1*(int)chiSqExpSlide.getValue();
        	curChiSq = chiInt*Math.pow(10,chiExp);
        	chiSqLabel.setText(chiString + String.valueOf(chiOutFormat.format(curChiSq)));
        	//if (!source.getValueIsAdjusting()) { updateOutCol(); }
        }
        
        if (source.equals(lapSlide)) {
        	curLap = (int)source.getValue();
        	lapLabel.setText(lapString + String.valueOf(curLap));
        	if (curLap == 0) { fitLap = false; } else { fitLap = true; }
        	if (!source.getValueIsAdjusting()) { updateGUI(); }
        }
        
        if (source.equals(angBaseSlide)) {
        	curAngleBase = source.getValue()*deltaSlope;
        	angBaseLabel.setText(angBaseString + String.valueOf(chiOutFormat.format(curAngleBase)));
        	if ((!source.getValueIsAdjusting()) & (!remote)) { filterLines(); }
        }
        
        if (source.equals(angTolSlide)) {
        	curAngleTol = source.getValue()*deltaSlope;
        	angTolLabel.setText(angTolString + String.valueOf(chiOutFormat.format(curAngleTol)));
        	if ((!source.getValueIsAdjusting()) & (!remote)) { filterLines(); }
        }
        
        if (source.equals(velocSlide)) {
        	curVeloc = source.getValue();
        	velocLabel.setText(velocString + String.valueOf((int)curVeloc) + " nm/s");
        	if (!source.getValueIsAdjusting()) { updateGUI(); }
        }
        
        if (source.equals(minStepSlide)) {
        	curMinPause = (int)source.getValue();
        	if (!remote) {
        	    double minTimeStep = (timeCol.data[1]-timeCol.data[0])*curMinPause*1000;
        		String minTimeStepString = " (" + String.valueOf((int)minTimeStep) + " ms)";
        		minStepLabel.setText(minStepString + String.valueOf((int)curMinPause) + minTimeStepString);
        	} else {
        		minStepLabel.setText(minStepString + String.valueOf((int)curMinPause));
        	}
        	if (!source.getValueIsAdjusting()) { updateGUI(); }
        }
        
        if (source.equals(disJSlide)) {
        	curDisJInt = (int)source.getValue();
        	curDisJ = curDisJInt*disJStep;
        	disJLabel.setText(disJDistString + String.valueOf(chiOutFormat.format(curDisJ*1000)) + " nm");
        	if ((!source.getValueIsAdjusting()) & (!remote)) { updatePlateaus(); }
        }        	
        		 
    } 
	
	public DataColumn getOutColumn() {
		//fitLines3();
		return outCol;
	}
	
	public void work () {
		outCol.clearLines();
		if (fitLap) {
			fitLines();
		} else {
			fitLines3();
		}
	}
	
	public void allSlidesEnabled () {
		chiSqSlide.setEnabled(true);
		chiSqExpSlide.setEnabled(true);
		lapSlide.setEnabled(true);
		velocSlide.setEnabled(true);
		minStepSlide.setEnabled(true);
		disJSlide.setEnabled(true); 
	}
	
	public void allSlidesDisabled () {
		chiSqSlide.setEnabled(false);
		chiSqExpSlide.setEnabled(false);
		lapSlide.setEnabled(false);
		velocSlide.setEnabled(false);
		minStepSlide.setEnabled(false);
		disJSlide.setEnabled(false); 
	}
	
	public void updateGUI() {
		if (fitLap) { 
			velocSlide.setEnabled(false);
			minStepSlide.setEnabled(false);
			disJSlide.setEnabled(false);
		} else {
			allSlidesEnabled();
		}
		
		if (!remote) {
			// plateaud functions
			if (plateaud) {
				eventHistoB.setEnabled(true);
				platsHistoB.setEnabled(true);
				filterPlatsB.setEnabled(true);
			} else {
				eventHistoB.setEnabled(false);
				platsHistoB.setEnabled(false);
				filterPlatsB.setEnabled(false);
			}
			
			// state of outCol functions
			if (outCol.distinctLines) {
				mergeB.setEnabled(true);
			} else {
				mergeB.setEnabled(false);
			}
		}
			
	}
	
	private void updateOutCol3 () {
		outCol.clearLines();
		fitLines3();
		rePlot();
	}
	
	private void updatePlateaus () {
		stepCol.clearLines();
		makePlateaus();
		rePlot();
	}
	
	private void makePlateauHisto() {
		if (plateaud) {
			File saveTo;
			Histogram plateauHisto = makeHistogram(stepCol);
			plateauSave.setDialogTitle("Save Step Histogram as...");
			int fileQuery = plateauSave.showSaveDialog(this);
			if (fileQuery == JFileChooser.APPROVE_OPTION) {
	            saveTo = plateauSave.getSelectedFile();
	            talkln ("Will save step data to " + saveTo.getAbsolutePath());
	        } else {
	            talkln ("Open command cancelled by user");
	            return;
	        }
	        plateauHisto.name = "";
	        plateauHisto.path = saveTo.getAbsolutePath();
			FileOps.mkHistogramFile(plateauHisto);
		}
	}
	
/*	private void makeEventHisto() {
		if (plateaud) {
			HistogramPlus [] hists = makeEventProfiles();
			FileOps.mkHistogramFile (hists);
		}
	} */
		
	public void fitLines() {
		outCol.distinctLines = true;
		boolean lastStep = false;
		clearSlope();
		int index = startIndex;
		int step = curLap+2;
		double chiSq = 0;
		double lastVal=0;
		double sigInc = 1.0;
		while (index < stopIndex-1-curLap) {
			chiSq = 0;
			step = curLap+2;
			Line fitL = new Line();
			while ((chiSq < curChiSq) & (!lastStep)) {
				if (index+step > stopIndex) { 
					step = stopIndex - index; 
					lastStep = true;
				}
				double [] rowVals = new double [step];
				double [] vals = new double [step];
				double [] sig = new double [step];
				for (int i=0;i<step;i++) {
					rowVals[i] = index + i;
					vals[i] = inCol.data[index+i];
					sig[i] = 1;//i*sigInc;
				}
				//if (index > 0) { vals[0] = lastVal; }		// don't use lastVal on first piece of data
				fitL = LinearFit.fit1D(rowVals,vals,step,sig,true);
				//lastVal = fitL.getVal(rowVals[step-1-lap]);
				chiSq = fitL.chiSq/step;
				//talkln ("pts on line = " + String.valueOf(step) + " ; chiSq = " + String.valueOf(chiSq));
				step++;
			}
			step -= 1; // take off that last step increment
			//talkln ("Line " + String.valueOf(fitL) + " ; " + fitL.reportLine());
			outCol.addLine(fitL);
			registerSlope(fitL);
			index += step-curLap;	// lap makes lines share endpts
		}
		outCol.linesToPoints();
		setDeltaSlope();
	}
	
	public void fitLines2() {
		outCol.distinctLines = true;
		boolean lastStep = false;
		clearSlope();
		int index = 0;
		int indexInc = 1;	// increment index by this much
		double chiSq = 0;
		double lastVal=0;
		double sigInc = 1.0;
		while (index < rowCt-1) {
			int step = 2;
			chiSq = 0;
			Line fitL = new Line();
			while ((chiSq < curChiSq) & (!lastStep)) {
				if (index+step > rowCt) { 
					step = rowCt - index; 
					lastStep = true;
				}
				double [] rowVals = new double [step];
				double [] vals = new double [step];
				double [] sig = new double [step];
				for (int i=0;i<step;i++) {
					rowVals[i] = index + i;
					vals[i] = inCol.data[index+i];
					sig[i] = 1;//i*sigInc;
				}
				//if (index > 0) { vals[0] = lastVal; }		// don't use lastVal on first piece of data
				fitL = LinearFit.fit1D(rowVals,vals,step,sig,true);
				//lastVal = fitL.getVal(rowVals[step-1-lap]);
				chiSq = fitL.chiSq/step;
				//talkln ("pts on line = " + String.valueOf(step) + " ; chiSq = " + String.valueOf(chiSq));
				step++;
			}
			//talkln ("Line " + String.valueOf(fitL) + " ; " + fitL.reportLine());
			outCol.addLine(fitL);
			registerSlope(fitL);
			index += indexInc;
		}
		outCol.linesToPoints();
		setDeltaSlope();
	}
	
	public void fitLines3() {	// with angle and ChiSq tolerance from the getgo
		outCol.distinctLines = true;
		int step,stepInc,yep,nope;
		boolean lastStep;
		double chiSq,slope;
		int index = startIndex;
		int indexInc = 1;	// increment index by this much
		int minStep = 1;	// line fitting step
		Line fitL = new Line();
		Line lineToAdd = null;
		while (index < stopIndex-1) {
			yep = index;
			nope = index;
			step = 16*minStep;
			stepInc = step;
			chiSq = 0;
			slope = 0;
			lineToAdd = null; 	// flag for adding line to outCol
			lastStep = false;
			while ((Math.abs(stepInc) > minStep) & (!lastStep)) {
				//talk ("trying step of " + String.valueOf(step));
				if (index+step > stopIndex) { 
					step = stopIndex - index; 
					lastStep = true;
				}
				fitL = makeLine(index,index+step,inCol);
				chiSq = fitL.chiSq/step;
				slope = fitL.getTimeSlope(timeCol);
				boolean stillGood = checkFit(chiSq,slope);
				if (stillGood) {
					//talkln ("...yep");
					yep = index + step;
					lineToAdd = fitL;	// keep a handle on the last good line
					//talkln ("slope is lineToAdd " + String.valueOf(slope*1e3) + " nm/s");
					if (nope == index) { // haven't failed yet
						stepInc = step;
					} else {
						stepInc = (int)((nope - yep)/2.0);
					}
				} else {
					//talkln("...nope");
					nope = index + step;
					stepInc = (int)(-1.0*(nope - yep)/2.0);
				}
				step += stepInc;
			}
			if (lineToAdd != null) { outCol.addLine(lineToAdd); }
			index += indexInc;
		}
		talkln ("fit " + String.valueOf(outCol.lineCt) + " lines");
		outCol.linesToPoints();
	}
	
	public boolean checkFit (double chi2, double slope) {
		if (chi2 > curChiSq) { return false; }
		double velocInNanosPerSec = Math.abs(slope)*1e3;
		if ( velocInNanosPerSec > curVeloc) { return false; }
		return true;
	}
	
	public void trimLines() {
		talkln ("trimming lines ...");
		RetObj retO;
		for (int i=0;i<outCol.lineCt-1;i++) {
			Line line1 = outCol.lines[i];
			Line line2 = outCol.lines[i+1];
			Pt3D pt1A = new Pt3D(line1.getVal(line1.start),(double)line1.start,0);
			Pt3D pt1B = new Pt3D(line1.getVal(line1.stop),(double)line1.stop,0);
			Pt3D pt2A = new Pt3D(line2.getVal(line2.start),(double)line2.start,0);
			Pt3D pt2B = new Pt3D(line2.getVal(line2.stop),(double)line2.stop,0);
			retO = lineSegmentIntersectTest  (pt1A,pt1B,pt2A,pt2B);
			if (retO.conPt1.x != -1) {
				int intPt = (int)(Math.ceil(retO.conPt1.y));
				line1.stop = intPt;
				line2.start = intPt;
			}
		}
	}
	
	public void mergeLines() {
		talkln ("merging lines ....");
		LineMap [] lineMaps = new LineMap[outCol.rows];	// one for each pt
		for (int i=0;i<outCol.rows;i++) { lineMaps[i] = new LineMap(); }	// initialize
		mapLinesToPts(lineMaps,outCol);
		outCol.distinctLines = false;
		double lastAve = 0;		// keep track of the last average
		boolean newAve = true;	// flag for resetting average count
		double curAveSum = 0;	// current average sum
		int curAveCt = 0;		// number of sums in current average
		for (int i=0;i<outCol.rows;i++) {
			double curVal = lineMaps[i].aveVal(i);
			if (Double.isNaN(curVal)) { 
				if (newAve) { 	// calculate new lastAve
					lastAve = curAveSum/curAveCt;
					curAveSum = 0;
					curAveCt = 0;
					newAve = false;
				}
				curVal = lastAve; 
			} else {
				newAve = true;
				curAveSum += curVal;
				curAveCt ++;
			}
			outCol.data[i] = curVal;
			//talkln ("value = " + String.valueOf(outCol.data[i]));
		}
	}
	
	public void mergeLinesToAveLine () {
		
	}
	
	public void makePlateaus () {
		if (outCol.lineCt == 0) { return; }
		talk ("Making Plateaus! ....");
		LineMap [] lineMaps = new LineMap[outCol.rows];	// one for each pt
		for (int i=0;i<outCol.rows;i++) { lineMaps[i] = new LineMap(); }	// initialize
		mapLinesToPts(lineMaps,outCol);
		disjointLineMaps(lineMaps);
		stepCol.distinctLines = true;
		boolean newPlateau = false;
		double aveSum = 0;
		double ave = 0;
		int startInd = -1;
		int stopInd = -1;
		double startVal = 0;
		double stopVal = 0;
		for (int i=0;i<lineMaps.length;i++) {
			LineMap curMap = lineMaps[i];
			if (curMap.getLineCt() > 0) {
				if (startInd == -1) { 
					startInd = i; 
					startVal = curMap.aveVal(i);
				}
				ave = curMap.aveVal(i);
				aveSum += ave;
			} else {
				if (startInd != -1) {
					stopInd = i-1;
					stopVal = ave;	// last ave value is stop value.
					newPlateau = true;
				}
			}
			if (newPlateau) {
				aveSum = aveSum/(stopInd-startInd+1);
				Plateau newP = new Plateau(startInd, stopInd, startVal, stopVal, aveSum, timeCol);
				stepCol.addLine(newP);
				// reset all flags and indices
				startInd = -1;
				stopInd = -1;
				newPlateau = false;
				aveSum = 0;
				ave = 0;
			}
		}
		setPlateauSteps();
		
		if ((!plateaud) & (!remote)) {
			parentFrame.analysisSet.addCol(stepCol);
			parentFrame.ammendSetUpdateGUI(parentFrame.analysisSet);
			plateaud = true;
		}
		talkln ("plateau# = " + String.valueOf(stepCol.lineCt));
			
	}
	
	public void disjointLineMaps(LineMap [] maps) {
		// internal check of each linemap...
		for (int i=0;i<rowCt;i++) {
			if (maps[i].bigJump(i)) { maps[i].lineCt = 0; } // zero the lines to break plateau in two
		}
		// check each pt against the next for a large change in average value
		for (int i=0;i<rowCt-1;i++) {
			if ((maps[i].getLineCt() > 0) & (maps[i+1].getLineCt() > 0)) {
				double thisIAve = maps[i].aveVal(i);
				double nextIAve = maps[i+1].aveVal(i+1);
				if (Math.abs(thisIAve-nextIAve) > curDisJ) { maps[i].lineCt = 0; }
			}
		}
		
	}
	
	public void filterPlateaus() {
		int i;
		
		/*// remove all plateaus below minimum pause duration
		i=0;
		double minPauseTime = (timeCol.data[1]-timeCol.data[0])*curMinPause;
		while (stepCol.lines[i] != null) {
			Plateau curP = (Plateau)stepCol.lines[i];
			if (curP.pauseTime < minPauseTime) {
				stepCol.removeLine(i);
			} else {
				i++;
			}
		} */
		
		
		if (stepCol.lineCt == 0) { return; }
		// check each plateau with the next to see if they are a minimum distance apart, if not join
		i=0;
		while ((stepCol.lines[i] != null) & (stepCol.lines[i+1] != null)) {
			Plateau p1 = (Plateau)stepCol.lines[i];
			Plateau p2 = (Plateau)stepCol.lines[i+1];
			double stopOf1 = p1.getStopVal();
			double startOf2 = p2.getStartVal();
			double dist = Math.abs(startOf2-stopOf1);
			if (dist < curDisJ) {
				p1.subsume(p2);
				stepCol.removeLine(i+1);
			} else {
				i++;
			}
		}
		
		// remove all plateaus below minimum pause duration
		i=0;
		double minPauseTime = (timeCol.data[1]-timeCol.data[0])*curMinPause;
		while (stepCol.lines[i] != null) {
			Plateau curP = (Plateau)stepCol.lines[i];
			if (curP.pauseTime < minPauseTime) {
				stepCol.removeLine(i);
			} else {
				i++;
			}
		}
		
		
		// set step distances
		setPlateauSteps();
	}
	
	public void setPlateauSteps() {
		Plateau curP=null,nextP=null;
		for (int i=0;i<stepCol.lineCt-1;i++) {
			curP = (Plateau)stepCol.lines[i];
			nextP = (Plateau)stepCol.lines[i+1];
			curP.nextStepSize = Math.abs(nextP.startVal - curP.stopVal);
		}
		nextP.nextStepSize = 0;
	}
		
	
	private DataSet getInSet() {
		if (inCol instanceof LineColumn) {
			LineColumn inLineCol = (LineColumn)inCol;
			return inLineCol.parCol.set;
		} else {
			return inCol.set;
		}
	}
	
	
	private DataColumn getPoly1Col() {
		return inSet.dataTable[5];
	}
	
	private DataColumn getPoly2Col() {
		return inSet.dataTable[6];
	}
	
	private DataColumn getPoly3Col() {
		return inSet.dataTable[7];
	}
	
	private DataColumn getNewTetCol() {
		return inSet.dataTable[8];
	}
	
	private DataColumn getCutTetCol() {
		return inSet.dataTable[9];
	}
	
	private DataColumn getPathXCol() {
		return inSet.dataTable[1];
	}
	
	private DataColumn getTetForceCol() {
		return inSet.dataTable[18];
	}
	
/*	public HistogramPlus [] makeEventProfiles () {
		DataSet curInSet = getInSet();
		return makeEventProfiles(curInSet);
	} */
	
	/*public HistogramPlus [] makeEventProfiles(DataSet curInSet) {
		this.inSet = curInSet;
		int stepsMinus = 60;
		int stepsPlus = 80;
		int numBins = stepsMinus+stepsPlus;
		double deltaT = timeCol.data[1]-timeCol.data[0];
		double startTime = -1*stepsMinus*deltaT;
		double stopTime = stepsPlus*deltaT;
		Plateau curPlat;
		// load event columns
		int numEvents = 7;
		eventCols = new DataColumn[numEvents];
		eventCols[0] = getPoly1Col();
		eventCols[1] = getPoly2Col();
		eventCols[2] = getPoly3Col();
		eventCols[3] = getNewTetCol();
		eventCols[4] = getCutTetCol();
		eventCols[5] = getPathXCol();
		eventCols[6] = getTetForceCol();
		
		// make histograms of event profiles
		eventHists = new HistogramPlus[numEvents];
		for (int k=0;k<numEvents;k++) {
			//talkln ("Event profile " + String.valueOf(k) + " is " + String.valueOf(eventCols[k].name));
			eventHists[k] = new HistogramPlus (numBins,startTime,stopTime,"/",eventCols[k].name,this);
			eventHists[k].reportAve = true;		// also report values / number of passes
		}
		
		for (int i=0;i<stepCol.lineCt;i++) {
			boolean stepOK = true;
			curPlat = (Plateau)stepCol.lines[i];
			// criteria for selecting plateaus
			if ((curPlat.pauseTime > 0.0075) | (curPlat.pauseTime < 0.0065)) { stepOK = false; }
			if (stepOK) {
				int sInd = curPlat.stop;
				// special values
				double zeroDispVal = eventCols[5].data[sInd];
				/*double tetherForceAve = 0;
				for (int k=0;k<numBins;k++) { 
					int index = k+sInd-stepsMinus;
					if ((index >= 0) & (index < eventCols[0].rows)) {
						tetherForceAve += eventCols[6].data[index];
					}
				} 
				tetherForceAve = tetherForceAve/numBins;
				// *****
				for (int k=0;k<numEvents;k++) { eventHists[k].addPass(); }
				for (int j=0;j<numBins;j++) {
					int curInd = j+sInd-stepsMinus;
					if ((curInd >= 0) & (curInd < eventCols[0].rows)) {
						for (int k=0;k<numEvents-2;k++) {
							eventHists[k].addValue(j, eventCols[k].data[curInd]);
						}
						// special work for path.x
						double dispFromZero = (eventCols[5].data[curInd]-zeroDispVal)/0.0027;
						eventHists[5].addValue(j,dispFromZero);
						
						// special work for tether force
						double tetFbyAve = eventCols[6].data[curInd]/1e-9;
						eventHists[6].addValue(j,tetFbyAve);
					}
				}	
			}
		}
		talkln ("used " + String.valueOf(eventHists[0].passCt) + " plateaus");
		return eventHists;
	} */
		
	
	public void plateauRunLines () {
		talkln("Making Plateau run lines...");
		int plateauNum = stepCol.lineCt;
		for (int i=0;i<plateauNum-1;i++) {
			int startId = stepCol.lines[i].stop;
			int stopId = stepCol.lines[i+1].start;
			Line runLine = makeLine(startId,stopId,inCol);
			stepCol.addLine(runLine);
		}
	}
	
	
	public void mapLinesToPts (LineMap [] lineMaps, LineColumn lCol) {
		for (int i=0;i<outCol.lineCt;i++) {
			Line curLine = outCol.lines[i];
			if (curLine.show) {	// only map this line if it is being shown (i.e. not being filtered)
				for (int j=curLine.start;j<=curLine.stop;j++) {
					lineMaps[j].addLine(curLine);
				}
			}
		}
	}
	
	public Line makeLine (int startInd, int stopInd, DataColumn dCol) {
		int step = stopInd - startInd;
		double [] rowVals = new double [step];
		double [] vals = new double [step];
		double [] sig = new double [step];
		for (int i=0;i<step;i++) {
			//talkln ("step=" + String.valueOf(step) + ", i=" + String.valueOf(i));
			rowVals[i] = startInd+i;
			vals[i] = dCol.data[startInd+i];
			sig[i] = 1;
		}
		return LinearFit.fit1D(rowVals,vals,step,sig,true);
	}
	
	public Histogram makeHistogram (LineColumn lCol) {
		return makeHistogram (lCol, null, null);
	}
	
	public Histogram makeHistogram (LineColumn lCol, String histPath, String histName) {
		talkln ("making histogram... ");
		int ptCt = 0;
		double minResStep = 0;
		int intervalCt = 250;
		double min = 0;		// min step (microns)
		double max = 0.10;	// max step (microns)
		double step = (max-min)/intervalCt;
		int [] hist = new int [intervalCt];
		for (int j=0;j<intervalCt;j++) { hist[j] = 0; } 	// initialize
		// assemble step values
		Plateau thisLine = new Plateau();
		boolean updateThisLine = true;
		boolean pausesOK = true;
		for (int i=0;i<lCol.lineCt-1;i++) {
			if (updateThisLine) { thisLine = (Plateau)lCol.lines[i]; }
			Plateau nextLine = (Plateau)lCol.lines[i+1];
			double startVal = thisLine.getStopVal();
			double stopVal = nextLine.getStartVal();
			if ((thisLine.pauseTime > 0.010) & (nextLine.pauseTime > 0.010)) { pausesOK = true; } else { pausesOK = false; }
			double stepVal = Math.abs(stopVal-startVal);
			int histIndex = (int)(Math.floor(stepVal/step));
			if (histIndex >= intervalCt) { histIndex = intervalCt-1; }
			if (stepVal > minResStep) {
				if (pausesOK) {
					hist[histIndex]++;
					ptCt++;
				}
				updateThisLine = true;
			} else {
				updateThisLine = false;
			}
		}
		// make intervals array and stepped hist array
		double [] vals = new double [intervalCt];
		double [] vals2 = new double [2*intervalCt+1];	
		int [] hist2 = new int [2*intervalCt+1];
		for (int j=0;j<intervalCt;j++) {
			vals[j] = j*step;
			vals2[2*j] = j*step;
			vals2[2*j+1] = j*step+step;
			hist2[2*j] = hist[j];
			hist2[2*j+1] = hist[j];
		}
		// zero the tail of the histogram
		vals2[2*intervalCt] = vals2[2*intervalCt-1];
		hist2[2*intervalCt] = 0;
		
		talkln (String.valueOf(ptCt) + " steps");
		
		//if (histPath == null) { histPath = "~/"; }
		if (histName == null) { histName = lCol.getName(); }
		
		Histogram newHisto = new Histogram(vals2,hist2,ptCt,histPath,histName);

		return newHisto;
	}
	
	public boolean meetsStepCrit (Plateau p1, Plateau p2, double stepSize) {
		double minPauseTime = 0;
		double minStepSize = curDisJ;
		
		if (p1.pauseTime < minPauseTime) { return false; }
		if (p2.pauseTime < minPauseTime) { return false; }
		if (stepSize < minStepSize) { return false; }
		
		return true;
	}
	
	public boolean meetsTetherCrit (Plateau p) {
		// filament number selection criteria
		int curTethers = (int)tethersCol.data[p.stop];
		if ((curTethers > 100) & (curTethers < 200)) { return true; }
		return false;
	}
		
	
	public HistogramPlus makeStepHistogram (LineColumn lCol) {
		talk ("making steps histogram... ");
		int binCt = 250;
		double min = 0;		// min step (m)
		double max = 0.05;	// max step (m)
		
		HistogramPlus stepsHist = new HistogramPlus(binCt,min,max,null,"-steps",this);
		
		Plateau curPlat = new Plateau();
		Plateau nextPlat = new Plateau();
		boolean updateCurPlat = true;
		for (int i=0;i<lCol.lineCt-1;i++) {
			if (updateCurPlat) { curPlat = (Plateau)lCol.lines[i]; }
			nextPlat = (Plateau)lCol.lines[i+1];
			double stepSize = Math.abs(curPlat.getStopVal() - nextPlat.getStartVal());
			if (meetsStepCrit(curPlat,nextPlat,stepSize)) {
				//if (meetsTetherCrit(curPlat)) {
					stepsHist.addValue(stepSize);
				//}
				updateCurPlat = true;
			} else {
				updateCurPlat = false;
			}
		}
		
		talkln (String.valueOf(stepsHist.ptCt) + " steps");
		
		return stepsHist;
	}
	
	public HistogramPlus makePauseHistogram (LineColumn lCol) {
		talk ("making pause histogram... ");
		int binCt = 100;
		double min = 0;		// min pause (secs)
		double max = 0.2;	// max pause (secs)
		
		HistogramPlus pauseHist = new HistogramPlus(binCt,min,max,null,"-pauses",this);
		
		for (int i=0;i<lCol.lineCt;i++) {
			Plateau curPlat = (Plateau)lCol.lines[i];
			//if (meetsTetherCrit(curPlat)) {
				pauseHist.addValue(curPlat.pauseTime);
			//}
		}
		
		talkln (String.valueOf(pauseHist.ptCt) + " pauses");
		
		return pauseHist;
	}
	
	public HistogramPlus makeVelocHistogram (LineColumn lCol) {
		talk ("making velocity histogram... " );
		double minRunTime = 0.050;	// minimum run time in seconds
		int binCt = 50;
		double min = 0;	// min velocity (
		double max = 2;	// max velocity (m/s)
		
		double minSpeed = curVeloc/1000;	// in m/s
		
		HistogramPlus velocHist = new HistogramPlus(binCt,min,max,null,"-velocities",this);
		
		double curV;
		for (int i=0;i<lCol.lineCt-1;i++) {
			Plateau curPlat = (Plateau)lCol.lines[i];
			Plateau nextPlat = (Plateau)lCol.lines[i+1];
			double runDuration = nextPlat.startTime - curPlat.stopTime;
			if (runDuration > minRunTime) {
				curV = (nextPlat.getStartVal() - curPlat.getStopVal())/runDuration;
				if ((curV > minSpeed)) { // & (meetsTetherCrit(curPlat))) {
					velocHist.addValue(curV);
				}
			}
		}
		
		talkln (String.valueOf(velocHist.ptCt) + " velocities");
		
		return velocHist;
	}
	
	public double sumPauseTime (LineColumn lCol) {
		double pauseSum = 0;
		for (int i=0;i<lCol.lineCt;i++) {
			Plateau curPlat = (Plateau)lCol.lines[i];
			pauseSum += curPlat.pauseTime;
		}
		
		return pauseSum;
	}
	
	
	private RetObj lineSegmentIntersectTest (Pt3D pt1A, Pt3D pt1B, Pt3D pt2A, Pt3D pt2B) {
		// this method implements an adaption of the "Faster Line Segment Intersection" technique of 
		// Franklin Antonio presented in "Graphics Gems III", ed David Kirk, IBM 1992 and "Intersection of 
		// Two Lines in Three-space" by Ronald Goldman in "Graphics Gems", 1990.
		// the points received define two line segments:  pt1A-pt1B and pt2A-pt2B
		// the object, RetObj, returned hold two Pt3D and one double
		Pt3D conPt1 = new Pt3D (-1,-1,-1);
		Pt3D conPt2 = new Pt3D (-1,-1,-1);
		double conDist = 0;
		Pt3D ray1 = Pt3D.Sub(pt1B,pt1A);
		Pt3D ray2 = Pt3D.Sub(pt2B,pt2A);
		Pt3D ray3 = Pt3D.Sub(pt2A,pt1A);
		Pt3D row1 = Pt3D.Cross (ray1,ray2);
		if ((row1.x == 0) & (row1.y == 0) & (row1.z == 0)) { 		// change this criterion to < some small #
			return new RetObj(conPt1,conPt2,conDist); 			// then stop 'cause the segments are parallel
		} else {
			double denom = Pt3D.Dot (row1,row1);
			double alpha = Pt3D.Dot(row1, Pt3D.Cross(ray3,ray2))/denom;
			if ((alpha >= 0) & (alpha <= 1)) {
				double beta = Pt3D.Dot(row1, Pt3D.Cross(ray3,ray1))/denom;
				if ((beta >= 0) & (beta <= 1)) {
					// if we've gotten this far we only need to check that the lines aren't skew... i.e. is there
					// one distinct intersection point or do we have the two points of closest approach?
					conPt1.add(pt1A, alpha, ray1);
					conPt2.add(pt2A, beta, ray2);
					conDist = Pt3D.ptDist (conPt1,conPt2);
				}
			}
		}
		return new RetObj(conPt1,conPt2,conDist);		
	}
	
	public void markSteps() {
		int ct = 0;
		double avVal = 0;
		for (int i=0;i<rowCt;i++) {
			if (ct == 0) { avVal = getAverage(i,avStub); }	
			outCol.data[i] = avVal;
			ct++;
			if (ct>=avStub) { ct = 0; }
		}
	}
	
	public double getAverage(int i, int stub) {
		double newAv = 0;
		for (int j=0;j<stub;j++) {
			if ((i+j) < rowCt) {
				newAv += inCol.data[i+j];
			}
		}
		newAv = newAv/stub;
		return newAv;
	}
	
	public void clearSlope () {
		maxSlope = 0;
		minSlope = 0;
	}
	
	public void registerSlope (Line curL) {
		if (curL.m < minSlope) { minSlope = curL.m; }
		if (curL.m > maxSlope) { maxSlope = curL.m; }
	}
		
	public void setDeltaSlope () {
		deltaSlope = (maxSlope-minSlope)/180;
	}
	
	public void filterLines () {
		double minSlopeCrit = curAngleBase-curAngleTol;
		double maxSlopeCrit = curAngleBase+curAngleTol;
		for (int i=0;i<outCol.lineCt;i++) {
			Line curL = outCol.lines[i];
			if ((curL.m < minSlopeCrit) | (curL.m > maxSlopeCrit)) { 
				curL.show = false; 
			} else {
				curL.show = true; 
			}
		}
	}
			
	private void makeTimePanel () {
		timePanel = new JPanel();
		timePanel.add(startTimeSlide);
		timePanel.add(startTimeLabel);
		timePanel.add(stopTimeSlide);
		timePanel.add(stopTimeLabel);
	}
	
	private void makeSliderPanel () {
		sliPanel = new JPanel();
		sliPanel.setLayout(new GridLayout(5,1));
		// make sub panel for two side by side sliders
		JPanel chiPanel = new JPanel();
		chiPanel.setLayout(new GridLayout(1,2));
		chiPanel.add(chiSqSlide);
		chiPanel.add(chiSqExpSlide);
		// *****
		sliPanel.add(chiPanel);
		sliPanel.add(lapSlide);
		sliPanel.add(velocSlide);
		sliPanel.add(minStepSlide);
		sliPanel.add(disJSlide);
		//sliPanel.add(angBaseSlide);
		//sliPanel.add(angTolSlide);
	}
	
	private void makeLabelPanel () {
		labPanel = new JPanel();
		labPanel.setLayout(new GridLayout(5,1));
		labPanel.add(chiSqLabel);
		labPanel.add(lapLabel);
		labPanel.add(velocLabel);
		labPanel.add(minStepLabel);
		labPanel.add(disJLabel);
		//labPanel.add(angBaseLabel);
		//labPanel.add(angTolLabel);
	}
	
	private void makeButtonPanel () {
		butPanel = new JPanel();
		JPanel workPanel = new JPanel();
		JPanel assundry1Panel = new JPanel();
		JPanel assundry2Panel = new JPanel();
		
		//work button
		JButton workB = new JButton("Work");
		workB.setActionCommand ("Work");
		workB.setToolTipText ("Calculate with current settings");
		//workB.setFont(Env.controlFont);
		workB.addActionListener(this);
		
		//trim button
		JButton trimB = new JButton("Trim");
		trimB.setActionCommand ("Trim");
		trimB.setToolTipText ("Trim Lines");
		trimB.setFont(Env.controlFont);
		trimB.addActionListener(this);
		
		//merge button
		mergeB = new JButton("Merge");
		mergeB.setActionCommand ("Merge");
		mergeB.setToolTipText ("Merge Lines");
		mergeB.setFont(Env.controlFont);
		mergeB.addActionListener(this);
		
		//plateau button
		JButton plateauB = new JButton("Plateaus");
		plateauB.setActionCommand ("Plateaus");
		plateauB.setToolTipText ("Make plateaus");
		plateauB.setFont(Env.controlFont);
		plateauB.addActionListener(this);
		
		//filter plateaus button
		filterPlatsB = new JButton("Filter Plateaus");
		filterPlatsB.setActionCommand ("Filter Plateaus");
		filterPlatsB.setToolTipText ("Filter current plateau list");
		filterPlatsB.setFont(Env.controlFont);
		filterPlatsB.addActionListener(this);
		
		//list plateaus button
		platsHistoB = new JButton("Histo Plateaus");
		platsHistoB.setActionCommand ("Plats Histo");
		platsHistoB.setToolTipText ("Histogram of plateaus");
		platsHistoB.setFont(Env.controlFont);
		platsHistoB.addActionListener(this);
		
		//events button
		eventHistoB = new JButton("Events");
		eventHistoB.setActionCommand ("Events Histo");
		eventHistoB.setToolTipText ("Histogram of events");
		eventHistoB.setFont(Env.controlFont);
		eventHistoB.addActionListener(this);
		
		// work panel setup
		workPanel.setLayout(new GridLayout(1,1));
		workPanel.add(workB);
		
		// assundry panel setup
		assundry1Panel.setLayout(new GridLayout(1,3));
		assundry2Panel.setLayout(new GridLayout(1,3));
		assundry1Panel.add(mergeB);
		assundry1Panel.add(trimB);
		assundry1Panel.add(plateauB);
		assundry2Panel.add(filterPlatsB);
		assundry2Panel.add(platsHistoB);
		assundry2Panel.add(eventHistoB);
		
		// button panel setup
		butPanel.setLayout(new GridLayout(3,1));
		butPanel.add(workPanel);
		butPanel.add(assundry1Panel);
		butPanel.add(assundry2Panel);
		
		
	}
		
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}

	
}